@@ -9,31 +9,36 @@ import numpy as np
 
 from . import util
 
+from hashlib import sha256
 from .constants import tol
 
-# how many significant figures to use for each field of the identifier
-id_sigfig = np.array([5,  # area
-                      10,  # euler number
-                      5,  # area/volume ratio
-                      2,  # convex/mesh area ratio
-                      2,  # convex area/volume ratio
-                      3])  # max radius squared / area
+# how many significant figures to use for each
+# field of the identifier based on hand-tuning
+id_sigfig = np.array(
+    [5,  # area
+     10,  # euler number
+     5,  # area/volume ratio
+     2,  # convex/mesh area ratio
+     2,  # convex area/volume ratio
+     3,  # max radius squared / area
+     1])  # signed triangle count for mirrored
 
 
 def identifier_simple(mesh):
     """
-    Return a basic identifier for a mesh, consisting of properties
-    that have been hand tuned to be somewhat robust to rigid
-    transformations and different tesselations.
+    Return a basic identifier for a mesh consisting of
+    properties that have been hand tuned to be somewhat
+    robust to rigid transformations and different
+    tesselations.
 
     Parameters
-    ----------
-    mesh : Trimesh object
+    ------------
+    mesh : trimesh.Trimesh
       Source geometry
 
     Returns
     ----------
-    identifier : (6,) float
+    identifier : (7,) float
       Identifying values of the mesh
     """
     # verify the cache once
@@ -45,7 +50,7 @@ def identifier_simple(mesh):
     with mesh._cache:
         # pre-allocate identifier so indexes of values can't move around
         # like they might if we used hstack or something else
-        identifier = np.zeros(6, dtype=np.float64)
+        identifier = np.zeros(7, dtype=np.float64)
         # avoid thrashing the cache unnecessarily
         mesh_area = mesh.area
         # start with properties that are valid regardless of watertightness
@@ -112,45 +117,54 @@ def identifier_simple(mesh):
             R2 = np.dot((vertices ** 2), [1, 1, 1]).max()
             identifier[5] = R2 / mesh_area
 
+    # mirrored meshes will look identical in terms of
+    # area, volume, etc: use a count of relative edge
+    # lengths to differentiate identical but mirrored meshes
+    # this doesn't work well on meshes with a small number of faces
+    # TODO : compare with "cross product of 2 orthogonal metrics"
+    # for a more principled way to detect mirrored meshes
+    if len(mesh.faces) > 50:
+        count = face_ordering(mesh).sum()
+        sign = float(count) / len(mesh.faces)
+        if abs(count) > 10 and abs(sign) > 0.02:
+            identifier[6] = sign
     return identifier
 
 
-def identifier_hash(identifier, sigfig=None):
+def identifier_hash(identifier):
     """
-    Hash an identifier array to a specified number of
-    significant figures.
+    Hash an identifier array in a way that is hand-tuned to be
+    somewhat robust to likely changes.
 
     Parameters
-    ----------
+    ------------
     identifier : (n,) float
       Vector of properties
-    sigfig : (n,) int
-      Number of sigfigs per property
 
     Returns
     ----------
-    md5 : str
-      MD5 hash of identifier
+    hash : (64,) str
+      A SHA256 of the identifier vector at hand-tuned precision.
     """
-    if sigfig is None:
-        sigfig = id_sigfig
 
     # convert identifier to integers and order of magnitude
-    as_int, multiplier = util.sigfig_int(identifier, sigfig)
+    as_int, multiplier = util.sigfig_int(
+        identifier, id_sigfig)
+
     # make all scales positive
     if (multiplier < 0).any():
         multiplier += np.abs(multiplier.min())
-    hashable = (as_int * (10 ** multiplier)).astype(np.int64)
-    md5 = util.md5_object(hashable)
-    return md5
+    data = (as_int * (10 ** multiplier)).astype(np.int64)
+    return sha256(data.tobytes()).hexdigest()
 
 
 def face_ordering(mesh):
     """
-    Triangles with three different length sides are
-    ordered in two ways:
-    [small edge, medium edge, large edge] (SML)
-    [small edge, large edge,  medium edge] (SLM)
+    Return the size-order of every face in the input mesh.
+
+    Triangles can be considered by the length order:
+      [small edge, medium edge, large edge] (SML)
+      [small edge, large edge,  medium edge] (SLM)
 
     This function returns [-1, 0, 1], depending on whether
     the triangle is SML or SLM, and 0 if M == L.
@@ -188,7 +202,7 @@ def face_ordering(mesh):
 
     # mark by sign but keep zero values zero
     order = np.zeros(len(norms), dtype=np.int64)
-    order[diff < tol.merge] = -1
+    order[diff < -tol.merge] = -1
     order[diff > tol.merge] = 1
 
     return order
