@@ -18,7 +18,8 @@ from .. import rendering
 from .. import resources
 from .. import transformations
 
-from ..caching import fast_hash
+from ..util import unique_name
+from ..caching import hash_fast
 from ..constants import log, tol
 
 
@@ -70,6 +71,7 @@ uint8 = np.dtype("<u1")
 def export_gltf(scene,
                 include_normals=None,
                 merge_buffers=False,
+                unitize_normals=False,
                 tree_postprocessor=None):
     """
     Export a scene object as a GLTF directory.
@@ -103,6 +105,7 @@ def export_gltf(scene,
     # create the header and buffer data
     tree, buffer_items = _create_gltf_structure(
         scene=scene,
+        unitize_normals=unitize_normals,
         include_normals=include_normals)
 
     # allow custom postprocessing
@@ -150,6 +153,7 @@ def export_gltf(scene,
 def export_glb(
         scene,
         include_normals=None,
+        unitize_normals=False,
         tree_postprocessor=None):
     """
     Export a scene as a binary GLTF (GLB) file.
@@ -179,6 +183,7 @@ def export_glb(
 
     tree, buffer_items = _create_gltf_structure(
         scene=scene,
+        unitize_normals=unitize_normals,
         include_normals=include_normals)
 
     # allow custom postprocessing
@@ -374,6 +379,7 @@ def load_glb(file_obj,
                            ignore_broken=ignore_broken,
                            merge_primitives=merge_primitives,
                            mesh_kwargs=mesh_kwargs)
+
     return kwargs
 
 
@@ -424,7 +430,7 @@ def _buffer_append(ordered, data):
       Index of buffer_items stored in
     """
     # hash the data to see if we have it already
-    hashed = fast_hash(data)
+    hashed = hash_fast(data)
     if hashed in ordered:
         # apparently they never implemented keys().index -_-
         return list(ordered.keys()).index(hashed)
@@ -456,12 +462,12 @@ def _data_append(acc, buff, blob, data):
     """
     # if we have data include that in the key
     as_bytes = data.tobytes()
-    if hasattr(data, 'fast_hash'):
+    if hasattr(data, 'hash_fast'):
         # passed a TrackedArray object
-        hashed = data.fast_hash()
+        hashed = data.hash_fast()
     else:
         # someone passed a vanilla numpy array
-        hashed = fast_hash(as_bytes)
+        hashed = hash_fast(as_bytes)
 
     if hashed in buff:
         blob['bufferView'] = list(buff.keys()).index(hashed)
@@ -547,9 +553,20 @@ def _mesh_to_material(mesh, metallic=0.0, rough=0.0):
     return material
 
 
+def _jsonify(blob):
+    """
+    Roundtrip a blob through json export-import cycle
+    skipping any internal keys.
+    """
+    return json.loads(util.jsonify(
+        {k: v for k, v in blob.items()
+         if not k.startswith('_')}))
+
+
 def _create_gltf_structure(scene,
                            include_normals=None,
-                           include_metadata=True):
+                           include_metadata=True,
+                           unitize_normals=None,):
     """
     Generate a GLTF header.
 
@@ -561,6 +578,8 @@ def _create_gltf_structure(scene,
       Include `scene.metadata` as `scenes/{idx}/extras/metadata`
     include_normals : bool
       Include vertex normals in output file?
+    unitize_normals : bool
+      Unitize all exported normals so as to pass GLTF validation
 
     Returns
     ---------------
@@ -588,14 +607,11 @@ def _create_gltf_structure(scene,
 
     if include_metadata and len(scene.metadata) > 0:
         try:
-            # collect extras from passed arguments and metadata
-            meta = scene.metadata.copy()
             # fail here if data isn't json compatible
-            util.jsonify(meta)
             # only export the extras if there is something there
-            tree['scenes'][0]['extras'] = meta
+            tree['scenes'][0]['extras'] = _jsonify(scene.metadata)
         except BaseException:
-            log.warning(
+            log.debug(
                 'failed to export scene metadata!', exc_info=True)
 
     # store materials as {hash : index} to avoid duplicates
@@ -617,6 +633,7 @@ def _create_gltf_structure(scene,
                 tree=tree,
                 buffer_items=buffer_items,
                 include_normals=include_normals,
+                unitize_normals=unitize_normals,
                 mat_hashes=mat_hashes)
         elif util.is_instance_named(geometry, "Path"):
             # add Path2D and Path3D objects
@@ -660,6 +677,7 @@ def _append_mesh(mesh,
                  tree,
                  buffer_items,
                  include_normals,
+                 unitize_normals,
                  mat_hashes):
     """
     Append a mesh to the scene structure and put the
@@ -677,12 +695,16 @@ def _append_mesh(mesh,
       Will have buffer appended with mesh data
     include_normals : bool
       Include vertex normals in export or not
+    unitize_normals : bool
+      Transform normals into unit vectors.
+      May be undesirable but will fail validators without this.
+
     mat_hashes : dict
       Which materials have already been added
     """
     # return early from empty meshes to avoid crashing later
     if len(mesh.faces) == 0 or len(mesh.vertices) == 0:
-        log.warning('skipping empty mesh!')
+        log.debug('skipping empty mesh!')
         return
     # convert mesh data to the correct dtypes
     # faces: 5125 is an unsigned 32 bit integer
@@ -716,13 +738,14 @@ def _append_mesh(mesh,
     # although that might be better, implicit works for 3DXML
     # https://github.com/KhronosGroup/glTF/tree/master/extensions
     try:
-        current['extras'] = json.loads(util.jsonify(
-            mesh.metadata))
+        # skip jsonify any metadata, skipping internal keys
+        current['extras'] = _jsonify(mesh.metadata)
+
         if mesh.units not in [None, 'm', 'meters', 'meter']:
             current["extras"]["units"] = str(mesh.units)
     except BaseException:
-        log.warning('metadata not serializable, dropping!',
-                    exc_info=True)
+        log.debug('metadata not serializable, dropping!',
+                  exc_info=True)
 
     # check to see if we have vertex or face colors
     # or if a TextureVisual has colors included as an attribute
@@ -781,6 +804,16 @@ def _append_mesh(mesh,
         (include_normals is None and
          'vertex_normals' in mesh._cache.cache)):
         # store vertex normals if requested
+        if unitize_normals:
+            normals = mesh.vertex_normals.copy()
+            norms = np.linalg.norm(normals, axis=1)
+            if not util.allclose(norms, 1.0, atol=1e-4):
+                normals /= norms.reshape((-1, 1))
+        else:
+            # we don't have to copy them since
+            # they aren't being altered
+            normals = mesh.vertex_normals
+
         acc_norm = _data_append(
             acc=tree['accessors'],
             buff=buffer_items,
@@ -788,7 +821,7 @@ def _append_mesh(mesh,
                   "count": len(mesh.vertices),
                   "type": "VEC3",
                   "byteOffset": 0},
-            data=mesh.vertex_normals.astype(float32))
+            data=normals.astype(float32))
         # add the reference for vertex color
         current["primitives"][0]["attributes"][
             "NORMAL"] = acc_norm
@@ -982,10 +1015,10 @@ def _append_path(path, name, tree, buffer_items):
     # if units are defined, store them as an extra:
     # https://github.com/KhronosGroup/glTF/tree/master/extensions
     try:
-        current["extras"] = json.loads(util.jsonify(path.metadata))
+        current["extras"] = _jsonify(path.metadata)
     except BaseException:
-        log.warning('failed to serialize metadata, dropping!',
-                    exc_info=True)
+        log.debug('failed to serialize metadata, dropping!',
+                  exc_info=True)
 
     if path.colors is not None:
         acc_color = _data_append(acc=tree['accessors'],
@@ -1122,7 +1155,7 @@ def _parse_materials(header, views, resolver=None):
     try:
         import PIL.Image
     except ImportError:
-        log.warning("unable to load textures without pillow!")
+        log.debug("unable to load textures without pillow!")
         return None
 
     # load any images
@@ -1139,7 +1172,7 @@ def _parse_materials(header, views, resolver=None):
                 # will get bytes from filesystem or base64 URI
                 blob = _uri_to_bytes(uri=img['uri'], resolver=resolver)
             else:
-                log.warning('unable to load image from: {}'.format(
+                log.debug('unable to load image from: {}'.format(
                     img.keys()))
                 continue
             # i.e. 'image/jpeg'
@@ -1182,49 +1215,6 @@ def _parse_materials(header, views, resolver=None):
     return materials
 
 
-def unique_name(start, contains):
-    """
-    Deterministically generate a unique name not
-    contained in a dict. Will create names of the
-    form "start_10" and increment accordingly.
-
-    Parameters
-    -----------
-    start : str
-      Initial guess for name
-    contains : dict, set, or list
-      Bundle of existing names we cannot use.
-
-    Returns
-    ---------
-    unique : str
-      A name that is not contained in `contains`
-    """
-    # exit early if name is not in bundle
-    if len(contains) == 0 or (len(start) > 0 and start not in contains):
-        return start
-
-    increment = 0
-    formatter = start + '_{}'
-    if len(start) > 0:
-        # split by our delimiter once
-        split = start.rsplit('_', 1)
-        if len(split) == 2 and split[1].isnumeric():
-            if split[0] not in contains:
-                return split[0]
-            # start incrementing from the passed value
-            increment = int(split[1])
-            formatter = split[0] + '_{}'
-
-    # if contains is empty we will only need to check once
-    for i in range(increment + 1, 2 + increment + len(contains)):
-        check = formatter.format(i)
-        if check not in contains:
-            return check
-
-    raise ValueError('unable to establish unique name!')
-
-
 def _read_buffers(header,
                   buffers,
                   mesh_kwargs,
@@ -1389,7 +1379,6 @@ def _read_buffers(header,
                             kwargs['faces'] = util.triangle_strips_to_faces([flat])
                         else:
                             kwargs["faces"] = access[p["indices"]].reshape((-1, 3))
-
                     else:
                         # indices are apparently optional and we are supposed to
                         # do the same thing as webGL drawArrays?
@@ -1403,7 +1392,7 @@ def _read_buffers(header,
                     visuals = None
                     if "material" in p:
                         if materials is None:
-                            log.warning('no materials! `pip install pillow`')
+                            log.debug('no materials! `pip install pillow`')
                         else:
                             uv = None
                             if "TEXCOORD_0" in attr:
@@ -1447,7 +1436,7 @@ def _read_buffers(header,
                     if len(custom) > 0:
                         kwargs["vertex_attributes"] = custom
                 else:
-                    log.warning('skipping primitive with mode %s!', mode)
+                    log.debug('skipping primitive with mode %s!', mode)
                     continue
                 # this should absolutely not be stomping on itself
                 assert name not in meshes
@@ -1771,8 +1760,11 @@ def _append_material(mat, tree, buffer_items, mat_hashes):
         result['name'] = as_pbr.name
 
     # if alphaMode is defined, export
-    if isinstance(as_pbr.alphaMode, str) and isinstance(as_pbr.alphaCutoff, float):
+    if isinstance(as_pbr.alphaMode, str):
         result['alphaMode'] = as_pbr.alphaMode
+
+    # if alphaCutoff is defined, export
+    if isinstance(as_pbr.alphaCutoff, float):
         result['alphaCutoff'] = as_pbr.alphaCutoff
 
     # if doubleSided is defined, export
