@@ -83,11 +83,11 @@ def load_obj(file_obj,
                          for k, v in material_kwargs.items()}
         except IOError:
             # usually the resolver couldn't find the asset
-            log.warning('unable to load materials from: {}'.format(mtl_path))
+            log.debug('unable to load materials from: {}'.format(mtl_path))
         except BaseException:
             # something else happened so log a warning
-            log.warning('unable to load materials from: {}'.format(mtl_path),
-                        exc_info=True)
+            log.debug('unable to load materials from: {}'.format(mtl_path),
+                      exc_info=True)
 
     # extract vertices from raw text
     v, vn, vt, vc = _parse_vertices(text=text)
@@ -162,7 +162,8 @@ def load_obj(file_obj,
             #  '31407 31406 31408',
             #  '32303/2469 32304/2469 32305/2469',
             log.debug('faces have mixed data, using slow fallback!')
-            faces, faces_tex, faces_norm = _parse_faces_fallback(face_lines)
+            faces, faces_tex, faces_norm = _parse_faces_fallback(
+                face_lines)
 
         if group_material:
             name = material
@@ -198,7 +199,7 @@ def load_obj(file_obj,
                 # want materials without UV coordinates
                 uv = vt[mask_vt]
             except BaseException:
-                log.warning('index failed on UV coordinates, skipping!')
+                log.debug('index failed on UV coordinates, skipping!')
                 uv = None
 
             # mask vertices and use new faces
@@ -208,15 +209,15 @@ def load_obj(file_obj,
         else:
             # otherwise just use unmasked vertices
             uv = None
-
             # check to make sure indexes are in bounds
             if tol.strict:
                 assert faces.max() < len(v)
-
             if vn is not None and np.shape(faces_norm) == faces.shape:
                 # do the crazy unmerging logic for split indices
                 new_faces, mask_v, mask_vn = unmerge_faces(
-                    faces, faces_norm, maintain_faces=maintain_order)
+                    faces,
+                    faces_norm,
+                    maintain_faces=maintain_order)
             else:
                 # generate the mask so we only include
                 # referenced vertices in every new mesh
@@ -234,21 +235,27 @@ def load_obj(file_obj,
             mesh.update({'faces': new_faces,
                          'vertices': v[mask_v].copy()})
 
-            # if colors and normals are OK save them
-            if vc is not None:
-                try:
-                    # may fail on a malformed color mask
-                    mesh['vertex_colors'] = vc[mask_v]
-                except BaseException:
-                    log.warning('failed to load vertex_colors',
-                                exc_info=True)
-            if mask_vn is not None:
-                try:
-                    # may fail on a malformed mask
-                    mesh['vertex_normals'] = vn[mask_vn]
-                except BaseException:
-                    log.warning('failed to load vertex_normals',
-                                exc_info=True)
+        # if colors and normals are OK save them
+        if vc is not None:
+            try:
+                # may fail on a malformed color mask
+                mesh['vertex_colors'] = vc[mask_v]
+            except BaseException:
+                log.warning('failed to load vertex_colors',
+                            exc_info=True)
+        if mask_vn is not None:
+            try:
+                # may fail on a malformed mask
+                normals = vn[mask_vn]
+                if normals.shape != mesh['vertices'].shape:
+                    raise ValueError('incorrect normals {} != {}'.format(
+                        str(normals.shape),
+                        str(mesh['vertices'].shape)))
+                mesh['vertex_normals'] = normals
+            except BaseException:
+                log.warning('failed to load vertex_normals',
+                            exc_info=True)
+
         visual = None
         if material in materials:
             # use the material with the UV coordinates
@@ -473,17 +480,19 @@ def _parse_faces_fallback(lines):
                      split[3],
                      split[0]]
         elif len_split > 4:
-            # triangulate polygon, as a triangles fan
-            r_split = []
-            r_split_append = r_split.append
-            for i in range(len(split) - 2):
-                r_split_append(split[0])
-                r_split_append(split[i + 1])
-                r_split_append(split[i + 2])
-            split = r_split
+            # triangulate polygon as a triangles fan
+            collect = []
+            # we need a flat list so append inside
+            # a list comprehension
+            collect_append = collect.append
+            [[collect_append(split[0]),
+              collect_append(split[i + 1]),
+              collect_append(split[i + 2])]
+             for i in range(len(split) - 2)]
+            split = collect
         else:
             log.warning(
-                'face need at least 3 elements (got {})! skipping!'.format(
+                'face needs more values 3>{} skipping!'.format(
                     len(split)))
             continue
 
