@@ -8,8 +8,11 @@ those stored in a DXF or SVG file.
 import numpy as np
 
 import copy
+import warnings
 import collections
 
+from hashlib import sha256
+
 from ..points import plane_fit
 from ..geometry import plane_transform
 from ..visual import to_rgba
@@ -28,23 +31,35 @@ from .. import exceptions
 from .. import transformations as tf
 
 from . import raster
-from . import repair
 from . import simplify
 from . import creation  # NOQA
-from . import polygons
 from . import segments  # NOQA
 from . import traversal
 
 from .exchange.export import export_path
 
-from scipy.spatial import cKDTree
-from shapely.geometry import Polygon
-
+# now import things which require non-minimal install of Trimesh
+# create a dummy module which will raise the ImportError
+# or other exception only when someone tries to use that function
+try:
+    from . import repair
+except BaseException as E:
+    repair = exceptions.ExceptionModule(E)
+try:
+    from . import polygons
+except BaseException as E:
+    polygons = exceptions.ExceptionModule(E)
+try:
+    from scipy.spatial import cKDTree
+except BaseException as E:
+    cKDTree = exceptions.closure(E)
+try:
+    from shapely.geometry import Polygon
+except BaseException as E:
+    Polygon = exceptions.closure(E)
 try:
     import networkx as nx
 except BaseException as E:
-    # create a dummy module which will raise the ImportError
-    # or other exception only when someone tries to use networkx
     nx = exceptions.ExceptionModule(E)
 
 
@@ -92,7 +107,8 @@ class Path(parent.Geometry):
             self.metadata.update(metadata)
 
         # cache will dump whenever self.crc changes
-        self._cache = caching.Cache(id_function=self.crc)
+        self._cache = caching.Cache(
+            id_function=self.__hash__)
 
         if process:
             # literally nothing will work if vertices
@@ -128,12 +144,11 @@ class Path(parent.Geometry):
           RGBA colors for each entity
         """
         # start with default colors
-        colors = np.ones((len(self.entities), 4))
-        colors = (colors * [100, 100, 100, 255]).astype(np.uint8)
-        # collect colors from entities
-        for i, e in enumerate(self.entities):
-            if hasattr(e, 'color') and e.color is not None:
-                colors[i] = to_rgba(e.color)
+        raw = [e.color for e in self.entities]
+        if not any(c is not None for c in raw):
+            return None
+
+        colors = np.array([to_rgba(c) for c in raw])
         # don't allow parts of the color array to be written
         colors.flags['WRITEABLE'] = False
         return colors
@@ -159,20 +174,6 @@ class Path(parent.Geometry):
         for c, e in zip(colors, self.entities):
             e.color = c
 
-    def colors_crc(self):
-        """
-        A CRC of the current entity colors.
-
-        Returns
-        -------
-        crc : int
-          CRC of the current entity colors
-        """
-        # first CRC the colors in every entity
-        target = caching.crc32(bytes().join(
-            e.color.tobytes() for e in self.entities))
-        return target
-
     @property
     def vertices(self):
         return self._vertices
@@ -211,42 +212,24 @@ class Path(parent.Geometry):
         layers : (len(entities), ) any
           Whatever is stored in each `entity.layer`
         """
-        # layer is a required property for entities
-        layers = [e.layer for e in self.entities]
-        return layers
+        # layer is a required meta-property for entities
+        return [e.layer for e in self.entities]
 
-    def crc(self):
+    def __hash__(self):
         """
-        A CRC of the current vertices and entities.
+        A hash of the current vertices and entities.
 
         Returns
         ------------
-        crc : int
-          CRC of entity points and vertices
+        hash : long int
+          Appended hashes
         """
-        # first CRC the points in every entity
-        target = caching.crc32(bytes().join(
-            e._bytes() for e in self.entities))
-        # XOR the CRC for the vertices
-        target ^= self.vertices.fast_hash()
-        return target
-
-    def md5(self):
-        """
-        An MD5 hash of the current vertices and entities.
-
-        Returns
-        ------------
-        md5 : str
-          Appended MD5 hashes
-        """
-        # first MD5 the points in every entity
-        target = '{}{}'.format(
-            util.md5_object(bytes().join(
-                e._bytes() for e in self.entities)),
-            self.vertices.md5())
-
-        return target
+        # get the hash of the trackedarray vertices
+        hashable = [hex(self.vertices.__hash__()).encode('utf-8')]
+        # get the bytes for each entity
+        hashable.extend(e._bytes() for e in self.entities)
+        # hash the combined result
+        return caching.hash_fast(b''.join(hashable))
 
     @caching.cache_decorator
     def paths(self):
@@ -258,8 +241,8 @@ class Path(parent.Geometry):
         paths : (n,) sequence of (*,) int
           Referencing self.entities
         """
-        paths = traversal.closed_paths(self.entities,
-                                       self.vertices)
+        paths = traversal.closed_paths(
+            self.entities, self.vertices)
         return paths
 
     @caching.cache_decorator
@@ -315,7 +298,8 @@ class Path(parent.Geometry):
 
         Returns
         --------
-        length: float, summed length of every entity
+        length : float
+          Summed length of every entity
         """
         length = float(sum(i.length(self.vertices)
                            for i in self.entities))
@@ -907,7 +891,7 @@ class Path3D(Path):
                     N *= np.sign(np.dot(N, normal))
                     N = normal
                 else:
-                    log.warning(
+                    log.debug(
                         "passed normal not used: {}".format(
                             normal.shape))
             # create a transform from fit plane to XY
@@ -1045,15 +1029,21 @@ class Path2D(Path):
 
         Parameters
         ------------
-        pitch:      float or (2,) float, length(s) in model space of pixel edges
-        origin:     (2,) float, origin position in model space
-        resolution: (2,) int, resolution in pixel space
-        fill:       bool, if True will return closed regions as filled
-        width:      int, if not None will draw outline this wide (pixels)
+        pitch : float or (2,) float
+          Length(s) in model space of pixel edges
+        origin : (2,) float
+          Origin position in model space
+        resolution : (2,) int
+          Resolution in pixel space
+        fill : bool
+          If True will return closed regions as filled
+        width : int
+          If not None will draw outline this wide (pixels)
 
         Returns
         ------------
-        raster: PIL.Image object, mode 1
+        raster : PIL.Image object, mode 1
+          Rasterized version of closed regions.
         """
         image = raster.rasterize(self,
                                  pitch=pitch,
@@ -1070,22 +1060,22 @@ class Path2D(Path):
 
         Parameters
         -----------
-        count   : int
-                    Number of points to return
-                    If there are multiple bodies, there will
-                    be up to count * bodies points returned
-        factor  : float
-                    How many points to test per loop
-                    IE, count * factor
+        count : int
+          Number of points to return
+          If there are multiple bodies, there will
+          be up to count * bodies points returned
+        factor : float
+          How many points to test per loop
+          IE, count * factor
         max_iter : int,
-                    Maximum number of intersection loops
-                    to run, total points sampled is
-                    count * factor * max_iter
+          Maximum number of intersection loops
+          to run, total points sampled is
+          count * factor * max_iter
 
         Returns
         -----------
         hit : (n, 2) float
-               Random points inside polygon
+          Random points inside polygon
         """
 
         poly = self.polygons_full
@@ -1102,6 +1092,15 @@ class Path2D(Path):
 
     @property
     def body_count(self):
+        """
+        Returns a count of the number of unconnected polygons that
+        may contain other curves but aren't contained themselves.
+
+        Returns
+        ---------
+        body_count : int
+          Number of unconnected independent polygons.
+        """
         return len(self.root)
 
     def to_3D(self, transform=None):
@@ -1111,13 +1110,14 @@ class Path2D(Path):
         Parameters
         -------------
         transform : (4, 4) float
-            If passed, will transform vertices.
-            If not passed and 'to_3D' is in metadata
-            that transform will be used.
+          If passed, will transform vertices.
+          If not passed and 'to_3D' is in self.metadata
+          that transform will be used.
 
         Returns
         -----------
-        path_3D: Path3D version of current path
+        path_3D : Path3D
+          3D version of current path
         """
         # if there is a stored 'to_3D' transform in metadata use it
         if transform is None and 'to_3D' in self.metadata:
@@ -1196,7 +1196,8 @@ class Path2D(Path):
 
         Returns
         ---------
-        area: float, total area of polygons minus interiors
+        area : float
+          Total area of polygons minus interiors
         """
         area = float(sum(i.area for i in self.polygons_full))
         return area
@@ -1392,7 +1393,16 @@ class Path2D(Path):
 
     def plot_entities(self, show=False, annotations=True, color=None):
         """
-        Plot the entities of the path, with no notion of topology
+        Plot the entities of the path with no notion of topology.
+
+        Parameters
+        ------------
+        show : bool
+          Open a window immediately or not
+        annotations : bool
+          Call an entities custom plot function.
+        color : str
+          Override entity colors and make them all this color.
         """
         import matplotlib.pyplot as plt
         # keep plot axis scaled the same
@@ -1419,7 +1429,7 @@ class Path2D(Path):
 
             fmt = eformat[e_key].copy()
             if color is not None:
-                # passed color will override other optons
+                # passed color will override other options
                 fmt['color'] = color
             elif hasattr(entity, 'color'):
                 # if entity has specified color use it
@@ -1435,42 +1445,64 @@ class Path2D(Path):
 
         Returns
         ---------
-        identifier: (5,) float, unique identifier
+        identifier : (5,) float
+          Unique identifier
         """
-        if len(self.polygons_full) != 1:
-            raise TypeError('Identifier only valid for single body')
-        return polygons.polygon_hash(self.polygons_full[0])
+        hasher = polygons.polygon_hash
+        target = self.polygons_full
+        if len(target) == 1:
+            return hasher(self.polygons_full[0])
+        elif len(target) == 0:
+            return np.zeros(5)
+
+        return np.sum([hasher(p) for p in target], axis=1)
 
     @caching.cache_decorator
-    def identifier_md5(self):
+    def identifier_hash(self):
         """
-        Return an MD5 of the identifier
+        Return a hash of the identifier.
+
+        Returns
+        ----------
+        hashed : (64,) str
+          SHA256 hash of the identifier vector.
         """
         as_int = (self.identifier * 1e4).astype(np.int64)
-        hashed = util.md5_object(as_int.tobytes(order='C'))
-        return hashed
+        return sha256(as_int.tobytes(order='C')).hexdigest()
+
+    @property
+    def identifier_md5(self):
+        warnings.warn(
+            '`geom.identifier_md5` is deprecated and will ' +
+            'be removed in October 2023: replace ' +
+            'with `geom.identifier_hash`',
+            DeprecationWarning)
+        return self.identifier_hash
 
     @property
     def path_valid(self):
         """
         Returns
         ----------
-        path_valid: (n,) bool, indexes of self.paths self.polygons_closed
-                         which are valid polygons
+        path_valid : (n,) bool
+          Indexes of self.paths self.polygons_closed
+          which are valid polygons.
         """
-        valid = [i is not None for i in self.polygons_closed]
-        valid = np.array(valid, dtype=bool)
+        valid = np.array(
+            [i is not None for i in self.polygons_closed],
+            dtype=bool)
         return valid
 
     @caching.cache_decorator
     def root(self):
         """
-        Which indexes of self.paths/self.polygons_closed are root curves.
-        Also known as 'shell' or 'exterior.
+        Which indexes of self.paths/self.polygons_closed
+        are root curves, also known as 'shell' or 'exterior.
 
         Returns
         ---------
-        root: (n,) int, list of indexes
+        root : (n,) int
+          List of indexes
         """
         populate = self.enclosure_directed  # NOQA
         return self._cache['root']
@@ -1478,7 +1510,12 @@ class Path2D(Path):
     @caching.cache_decorator
     def enclosure(self):
         """
-        Networkx Graph object of polygon enclosure.
+        Undirected graph object of polygon enclosure.
+
+        Returns
+        -----------
+        enclosure : networkx.Graph
+          Enclosure graph of self.polygons by index.
         """
         with self._cache:
             undirected = self.enclosure_directed.to_undirected()
@@ -1487,7 +1524,13 @@ class Path2D(Path):
     @caching.cache_decorator
     def enclosure_directed(self):
         """
-        Networkx DiGraph of polygon enclosure
+        Directed graph of polygon enclosure.
+
+        Returns
+        ----------
+        enclosure_directed : networkx.DiGraph
+          Directed graph: child nodes are fully
+          contained by their parent node.
         """
         root, enclosure = polygons.enclosure_tree(self.polygons_closed)
         self._cache['root'] = root
@@ -1501,7 +1544,8 @@ class Path2D(Path):
 
         Returns
         ----------
-        corresponding: dict, {index of self.paths of shell : [indexes of holes]}
+        corresponding : dict
+          {index of self.paths of shell : [indexes of holes]}
         """
         pairs = [(r, self.connected_paths(r, include_self=False))
                  for r in self.root]
