@@ -10,7 +10,14 @@ from . import util
 
 from .grouping import group_min
 from .constants import tol, log_time
-from .triangles import closest_point as closest_point_corresponding
+from .triangles import closest_point as _corresponding
+from .triangles import points_to_barycentric
+
+try:
+    from scipy.spatial import cKDTree
+except BaseException as E:
+    from .exceptions import closure
+    cKDTree = closure(E)
 
 
 def nearby_faces(mesh, points):
@@ -26,12 +33,15 @@ def nearby_faces(mesh, points):
 
     Parameters
     ----------
-    mesh : Trimesh object
-    points : (n,3) float , points in space
+    mesh : trimesh.Trimesh
+      Mesh to query.
+    points : (n, 3) float
+       Points in space
 
     Returns
     -----------
-    candidates : (points,) int, sequence of indexes for mesh.faces
+    candidates : (points,) int
+      Sequence of indexes for mesh.faces
     """
     points = np.asanyarray(points, dtype=np.float64)
     if not util.is_shape(points, (-1, 3)):
@@ -40,7 +50,7 @@ def nearby_faces(mesh, points):
     # an r-tree containing the axis aligned bounding box for every triangle
     rtree = mesh.triangles_tree
     # a kd-tree containing every vertex of the mesh
-    kdtree = mesh.kdtree
+    kdtree = cKDTree(mesh.vertices[mesh.referenced_vertices])
 
     # query the distance to the nearest vertex to get AABB of a sphere
     distance_vertex = kdtree.query(points)[0].reshape((-1, 1))
@@ -91,7 +101,7 @@ def closest_point_naive(mesh, points):
 
     # create a giant tiled array of each point tiled len(triangles) times
     points_tiled = np.tile(points, (1, len(triangles)))
-    on_triangle = np.array([closest_point_corresponding(
+    on_triangle = np.array([_corresponding(
         triangles, i.reshape((-1, 3))) for i in points_tiled])
 
     # distance squared
@@ -114,7 +124,7 @@ def closest_point(mesh, points):
 
     Parameters
     ----------
-    mesh   : trimesh.Trimesh
+    mesh : trimesh.Trimesh
       Mesh to query
     points : (m, 3) float
       Points in space
@@ -124,7 +134,7 @@ def closest_point(mesh, points):
     closest : (m, 3) float
       Closest point on triangles for each point
     distance : (m,)  float
-      Distance
+      Distance to mesh.
     triangle_id : (m,) int
       Index of triangle containing closest point
     """
@@ -141,13 +151,15 @@ def closest_point(mesh, points):
     # create the corresponding list of triangles
     # and query points to send to the closest_point function
     all_candidates = np.concatenate(candidates)
+
     num_candidates = list(map(len, candidates))
     tile_idxs = np.repeat(np.arange(len(points)), num_candidates)
     query_point = points[tile_idxs, :]
+
     query_tri = triangles[all_candidates]
 
     # do the computation for closest point
-    query_close = closest_point_corresponding(query_tri, query_point)
+    query_close = _corresponding(query_tri, query_point)
     query_group = np.cumsum(num_candidates)[:-1]
 
     # vectors and distances for
@@ -160,7 +172,8 @@ def closest_point(mesh, points):
     idxs = np.int32([qd.argsort()[:2] if len(qd) > 1 else [0, 0] for qd in qds])
     idxs[1:] += query_group.reshape(-1, 1)
 
-    # distances and traingle ids for best two candidates
+    # points, distances and triangle ids for best two candidates
+    two_points = query_close[idxs]
     two_dists = query_distance[idxs]
     two_candidates = all_candidates[idxs]
 
@@ -192,6 +205,8 @@ def closest_point(mesh, points):
     # correct triangle ids where necessary
     # closest point and distance remain valid
     result_tid[c_mask] = two_candidates[c_mask, c_idxs]
+    result_distance[c_mask] = two_dists[c_mask, c_idxs]
+    result_close[c_mask] = two_points[c_mask, c_idxs]
 
     # we were comparing the distance squared so
     # now take the square root in one vectorized operation
@@ -210,12 +225,15 @@ def signed_distance(mesh, points):
 
     Parameters
     -----------
-    mesh   : Trimesh object
-    points : (n,3) float, list of points in space
+    mesh : trimesh.Trimesh
+      Mesh to query.
+    points : (n, 3) float
+      Points in space
 
     Returns
     ----------
-    signed_distance : (n,3) float, signed distance from point to mesh
+    signed_distance : (n,) float
+      Signed distance from point to mesh
     """
     # make sure we have a numpy array
     points = np.asanyarray(points, dtype=np.float64)
@@ -229,15 +247,60 @@ def signed_distance(mesh, points):
     if not nonzero.any():
         return distance
 
-    inside = mesh.ray.contains_points(points[nonzero])
-    sign = (inside.astype(int) * 2) - 1
+    # For closest points that project directly in to the triangle, compute sign from
+    # triangle normal Project each point in to the closest triangle plane
+    nonzero = np.where(nonzero)[0]
+    normals = mesh.face_normals[triangle_id]
+    projection = (points[nonzero] -
+                  (normals[nonzero].T * np.einsum(
+                      "ij,ij->i",
+                      points[nonzero] - closest[nonzero],
+                      normals[nonzero])).T)
+
+    # Determine if the projection lies within the closest triangle
+    barycentric = points_to_barycentric(
+        mesh.triangles[triangle_id[nonzero]],
+        projection)
+    ontriangle = ~((
+        (barycentric < -tol.merge) | (barycentric > 1 + tol.merge)
+    ).any(axis=1))
+
+    # Where projection does lie in the triangle, compare vector to projection to the
+    # triangle normal to compute sign
+    sign = np.sign(np.einsum(
+        "ij,ij->i",
+        normals[nonzero[ontriangle]],
+        points[nonzero[ontriangle]] - projection[ontriangle]))
+    distance[nonzero[ontriangle]] *= -1.0 * sign
+
+    # For all other triangles, resort to raycasting against the entire mesh
+    inside = mesh.ray.contains_points(points[nonzero[~ontriangle]])
+    sign = (inside.astype(int) * 2) - 1.0
 
     # apply sign to previously computed distance
-    distance[nonzero] *= sign
+    distance[nonzero[~ontriangle]] *= sign
 
     return distance
 
 
+class NearestQueryResult:
+    """
+    Stores the nearest points and attributes for nearest points queries.
+    """
+
+    def __init__(self):
+        self.nearest = None
+        self.distances = None
+        self.normals = None
+        self.triangle_indices = None
+        self.barycentric_coordinates = None
+        self.interpolated_normals = None
+        self.vertex_indices = None
+
+    def has_normals(self):
+        return self.normals is not None or self.interpolated_normals is not None
+
+
 class ProximityQuery(object):
     """
     Proximity queries for the current mesh.
@@ -258,9 +321,12 @@ class ProximityQuery(object):
 
         Returns
         ----------
-        closest     : (m,3) float, closest point on triangles for each point
-        distance    : (m,)  float, distance
-        triangle_id : (m,)  int, index of closest triangle for each point
+        closest : (m, 3) float
+          Closest point on triangles for each point
+        distance : (m,) float
+          Distance to surface
+        triangle_id : (m,) int
+          Index of closest triangle for each point.
         """
         return closest_point(mesh=self._mesh,
                              points=points)
@@ -271,12 +337,15 @@ class ProximityQuery(object):
 
         Parameters
         ----------
-        points : (n,3) float, list of points in space
+        points : (n, 3) float
+          Points in space
 
         Returns
         ----------
-        distance  : (n,) float, distance from source point to vertex
-        vertex_id : (n,) int, index of mesh.vertices which is closest
+        distance : (n,) float
+           Distance from source point to vertex.
+        vertex_id : (n,) int
+          Index of mesh.vertices for closest vertex.
         """
         tree = self._mesh.kdtree
         return tree.query(points)
@@ -291,11 +360,13 @@ class ProximityQuery(object):
 
         Parameters
         -----------
-        points : (n,3) float, list of points in space
+        points : (n, 3) float
+          Points in space
 
         Returns
         ----------
-        signed_distance : (n,3) float, signed distance from point to mesh
+        signed_distance : (n,) float
+          Signed distance from point to mesh.
         """
         return signed_distance(self._mesh, points)
 
@@ -307,12 +378,15 @@ def longest_ray(mesh, points, directions):
 
     Parameters
     -----------
-    points : (n,3) float, list of points in space
-    directions : (n,3) float, directions of rays
+    points : (n, 3) float
+      Points in space.
+    directions : (n, 3) float
+      Directions of rays.
 
     Returns
     ----------
-    signed_distance : (n,) float, length of rays
+    signed_distance : (n,) float
+      Length of rays.
     """
     points = np.asanyarray(points, dtype=np.float64)
     if not util.is_shape(points, (-1, 3)):
@@ -366,16 +440,20 @@ def max_tangent_sphere(mesh,
 
     Parameters
     ----------
-    points : (n,3) float, list of points in space
-    inwards : bool, whether to have the sphere inside or outside the mesh
-    normals : (n,3) float, normals of the mesh at the given points
-              None, compute this automatically.
+    points : (n, 3) float
+      Points in space.
+    inwards : bool
+      Whether to have the sphere inside or outside the mesh.
+    normals : (n, 3) float or None
+      Normals of the mesh at the given points
+      if is None computed automatically.
 
     Returns
     ----------
-    centers : (n,3) float, centers of spheres
-    radii : (n,) float, radii of spheres
-
+    centers : (n,3) float
+      Centers of spheres
+    radii : (n,) float
+      Radii of spheres
     """
     points = np.asanyarray(points, dtype=np.float64)
     if not util.is_shape(points, (-1, 3)):
@@ -397,7 +475,7 @@ def max_tangent_sphere(mesh,
     # Find initial tangent spheres
     distances = longest_ray(mesh, points, normals)
     radii = distances * 0.5
-    not_converged = np.ones(len(points), dtype=np.bool)  # boolean mask
+    not_converged = np.ones(len(points), dtype=bool)  # boolean mask
 
     # If ray is infinite, find the vertex which is furthest from our point
     # when projected onto the ray. I.e. find v which maximises
@@ -473,16 +551,21 @@ def thickness(mesh,
 
     Parameters
     ----------
-    points : (n,3) float, list of points in space
-    exterior : bool, whether to compute the exterior thickness
-                     (a.k.a. reach)
-    normals : (n,3) float, normals of the mesh at the given points
-              None, compute this automatically.
-    method : string, one of 'max_sphere' or 'ray'
+    points : (n, 3) float
+      Points in space
+    exterior : bool
+      Whether to compute the exterior thickness
+      (a.k.a. reach)
+    normals : (n, 3) float
+      Normals of the mesh at the given points
+      If is None computed automatically.
+    method : string
+      One of 'max_sphere' or 'ray'
 
     Returns
     ----------
-    thickness : (n,) float, thickness
+    thickness : (n,) float
+      Thickness at given points.
     """
     points = np.asanyarray(points, dtype=np.float64)
     if not util.is_shape(points, (-1, 3)):
