@@ -1,22 +1,21 @@
-import hashlib
 import numpy as np
 import collections
 
+from . import cameras
+from . import lighting
+
 from .. import util
 from .. import units
 from .. import convex
 from .. import caching
 from .. import grouping
 from .. import transformations
-
 from .. import bounds as bounds_module
 
+from ..util import unique_name
 from ..exchange import export
 from ..parent import Geometry3D
 
-from . import cameras
-from . import lighting
-
 from .transforms import SceneGraph
 
 
@@ -63,7 +62,7 @@ class Scene(Geometry3D):
         self.graph = SceneGraph(base_frame=base_frame)
 
         # create our cache
-        self._cache = caching.Cache(id_function=self.md5)
+        self._cache = caching.Cache(id_function=self.__hash__)
 
         # add passed geometry to scene
         self.add_geometry(geometry)
@@ -142,9 +141,9 @@ class Scene(Geometry3D):
                 extras=extras) for value in geometry]
         elif isinstance(geometry, dict):
             # if someone passed us a dict of geometry
-            for key, value in geometry.items():
-                self.add_geometry(value, geom_name=key, extras=extras)
-            return
+            return {k: self.add_geometry(v, geom_name=k, extras=extras)
+                    for k, v in geometry.items()}
+
         elif isinstance(geometry, Scene):
             # concatenate current scene with passed scene
             concat = self + geometry
@@ -154,9 +153,11 @@ class Scene(Geometry3D):
             # replace graph data with concatenated graph
             self.graph.transforms = concat.graph.transforms
             return
-        elif not hasattr(geometry, 'vertices'):
+
+        if not hasattr(geometry, 'vertices'):
             util.log.warning('unknown type ({}) added to scene!'.format(
                 type(geometry).__name__))
+            return
 
         # get or create a name to reference the geometry by
         if geom_name is not None:
@@ -171,25 +172,20 @@ class Scene(Geometry3D):
             # try to create a simple name
             name = 'geometry_' + str(len(self.geometry))
 
-        # if its already taken add a unique random string to it
-        if name in self.geometry:
-            name += ':' + util.unique_id().upper()
-
+        # if its already taken use our unique name logic
+        name = unique_name(start=name, contains=self.geometry.keys())
         # save the geometry reference
         self.geometry[name] = geometry
 
         # create a unique node name if not passed
         if node_name is None:
             # if the name of the geometry is also a transform node
-            if name in self.graph.nodes:
-                # a random unique identifier
-                unique = util.unique_id()
-                # geometry name + UUID
-                node_name = name + '_' + unique.upper()
-                assert node_name not in self.graph.nodes
-            else:
-                # otherwise make the transform node name the same as the geom
-                node_name = name
+            # which graph nodes already exist
+            existing = self.graph.transforms.node_data.keys()
+            # find a name that isn't contained already starting
+            # at the name we have
+            node_name = unique_name(name, existing)
+            assert node_name not in existing
 
         if transform is None:
             # create an identity transform from parent_node
@@ -234,31 +230,7 @@ class Scene(Geometry3D):
             if util.is_instance_named(geometry, 'Trimesh'):
                 geometry.visual = ColorVisuals(mesh=geometry)
 
-    def md5(self):
-        """
-        MD5 of scene which will change when meshes or
-        transforms are changed
-
-        Returns
-        --------
-        hashed : str
-          MD5 hash of scene
-        """
-        # start with transforms hash
-        return hashlib.md5(self._hashable()).hexdigest()
-
-    def crc(self):
-        """
-        Get a CRC of the current geometry and graph information.
-
-        Returns
-        ---------
-        crc : int
-          Hash of current graph and geometry.
-        """
-        return caching.crc32(self._hashable())
-
-    def _hashable(self):
+    def __hash__(self):
         """
         Return information about scene which is hashable.
 
@@ -267,15 +239,15 @@ class Scene(Geometry3D):
         hashable : str
           Data which can be hashed.
         """
+        # avoid accessing attribute in tight loop
+        geometry = self.geometry
         # start with the last modified time of the scene graph
-        hashable = [self.graph.modified()]
-        # crc is an abstractmethod for all Geometry3D
-        # objects so everything should really have it
-        hashable.extend(str(i.crc()) for i in
-                        self.geometry.values()
-                        if hasattr(i, 'crc'))
-        # crc requires bytes so encode to utf-8
-        return ':'.join(sorted(hashable)).encode('utf-8')
+        hashable = [hex(hash(self.graph))]
+        # take the re-hex string of the hash
+        hashable.extend(hex(geometry[k].__hash__()) for k in
+                        sorted(geometry.keys()))
+        return caching.hash_fast(
+            ''.join(hashable).encode('utf-8'))
 
     @property
     def is_empty(self):
@@ -425,12 +397,9 @@ class Scene(Geometry3D):
         # get the area of every geometry that has an area property
         areas = {n: g.area for n, g in self.geometry.items()
                  if hasattr(g, 'area')}
-        # get the name of every geometry instance in the scene
-        geoms = [self.graph[n][1] for n in
-                 self.graph.nodes_geometry]
-        # sum the area for every instanced geometry
-        area = sum(areas[n] for n in geoms if n in geoms)
-        return area
+        # sum the area including instancing
+        return sum((areas.get(self.graph[n][1], 0.0) for n in
+                    self.graph.nodes_geometry), 0.0)
 
     @caching.cache_decorator
     def triangles(self):
@@ -484,14 +453,14 @@ class Scene(Geometry3D):
     @caching.cache_decorator
     def geometry_identifiers(self):
         """
-        Look up geometries by identifier MD5
+        Look up geometries by identifier hash
 
         Returns
         ---------
         identifiers : dict
-          {Identifier MD5: key in self.geometry}
+          {Identifier hash: key in self.geometry}
         """
-        identifiers = {mesh.identifier_md5: name
+        identifiers = {mesh.identifier_hash: name
                        for name, mesh in self.geometry.items()}
         return identifiers
 
@@ -512,10 +481,10 @@ class Scene(Geometry3D):
         if len(self.geometry) == 0:
             return []
 
-        # geometry name : md5 of mesh
-        hashes = {k: int(m.identifier_md5, 16)
+        # geometry name : hash of mesh
+        hashes = {k: int(m.identifier_hash, 16)
                   for k, m in self.geometry.items()
-                  if hasattr(m, 'identifier_md5')}
+                  if hasattr(m, 'identifier_hash')}
 
         # bring into local scope for loop
         graph = self.graph
@@ -1241,7 +1210,8 @@ def append_scenes(iterable, common=['world'], base_frame='world'):
         # if a node is consumed and isn't one of the nodes
         # we're going to hold common between scenes remap it
         if node not in common and node in consumed:
-            name = str(node) + '-' + util.unique_id().upper()
+            # generate a name not in consumed
+            name = node + util.unique_id()
             map_node[node] = name
             node = name
 
@@ -1264,10 +1234,7 @@ def append_scenes(iterable, common=['world'], base_frame='world'):
         map_geom = {}
         for k, v in s.geometry.items():
             # if a geometry already exists add a UUID to the name
-            if k in geometry:
-                name = str(k) + '-' + util.unique_id().upper()
-            else:
-                name = k
+            name = unique_name(start=k, contains=geometry.keys())
             # store name mapping
             map_geom[k] = name
             # store geometry with new name
