@@ -18,7 +18,6 @@ import base64
 import random
 import shutil
 import logging
-import hashlib
 import zipfile
 import tempfile
 import collections
@@ -39,9 +38,11 @@ if PY3:
     basestring = str
     # Python 3
     from io import BytesIO, StringIO
+    from shutil import which  # noqa
 else:
     # Python 2
     from StringIO import StringIO
+    from distutils.spawn import find_executable as which  # noqa
     # monkey patch StringIO so `with` statements work
     StringIO.__enter__ = lambda a: a
     StringIO.__exit__ = lambda a, b, c, d: a.close()
@@ -567,7 +568,7 @@ try:
     # only included in recent-ish version of numpy
     multi_dot = np.linalg.multi_dot
 except AttributeError:
-    log.warning('np.linalg.multi_dot not available, using fallback')
+    log.debug('np.linalg.multi_dot not available, using fallback')
 
     def multi_dot(arrays):
         """
@@ -858,35 +859,6 @@ def decimal_to_digits(decimal, min_digits=None):
     return digits
 
 
-def hash_file(file_obj,
-              hash_function=hashlib.md5):
-    """
-    Get the hash of an open file-like object.
-
-    Parameters
-    ------------
-    file_obj: file like object
-    hash_function: function to use to hash data
-
-    Returns
-    ---------
-    hashed: str, hex version of result
-    """
-    # before we read the file data save the current position
-    # in the file (which is probably 0)
-    file_position = file_obj.tell()
-    # create an instance of the hash object
-    hasher = hash_function()
-    # read all data from the file into the hasher
-    hasher.update(file_obj.read())
-    # get a hex version of the result
-    hashed = hasher.hexdigest()
-    # return the file object to its original position
-    file_obj.seek(file_position)
-
-    return hashed
-
-
 def attach_to_log(level=logging.DEBUG,
                   handler=None,
                   loggers=None,
@@ -1674,9 +1646,10 @@ def jsonify(obj, **kwargs):
 
     Parameters
     --------------
-    obj : JSON-serializable blob
-    **kwargs :
-        Passed to json.dumps
+    obj : list, dict
+      A JSON-serializable blob
+    kwargs : dict
+      Passed to json.dumps
 
     Returns
     --------------
@@ -2347,7 +2320,7 @@ def decode_text(text, initial='utf-8'):
         # try to detect the encoding of the file
         detect = chardet.detect(text)
         # warn on files that aren't UTF-8
-        log.warning(
+        log.debug(
             'Data not {}! Trying {} (confidence {})'.format(
                 initial,
                 detect['encoding'],
@@ -2409,3 +2382,54 @@ def is_ccw(points):
     ccw = area < 0
 
     return ccw
+
+
+def unique_name(start, contains):
+    """
+    Deterministically generate a unique name not
+    contained in a dict, set or other grouping with
+    `__includes__` defined. Will create names of the
+    form "start_10" and increment accordingly.
+
+    Parameters
+    -----------
+    start : str
+      Initial guess for name.
+    contains : dict, set, or list
+      Bundle of existing names we can *not* use.
+
+    Returns
+    ---------
+    unique : str
+      A name that is not contained in `contains`
+    """
+    # exit early if name is not in bundle
+    if len(contains) == 0 or (len(start) > 0 and start not in contains):
+        return start
+
+    # start checking with zero index
+    increment = 0
+    formatter = start + '_{}'
+
+    if len(start) > 0:
+        # split by our delimiter once
+        split = start.rsplit('_', 1)
+        if len(split) == 2:
+            try:
+                # start incrementing from the existing trailing value
+                # if it is not an integer this will fail
+                increment = int(split[1])
+                # include the first split value
+                formatter = split[0] + '_{}'
+            except BaseException:
+                pass
+
+    # if contains is empty we will only need to check once
+    for i in range(increment + 1, 2 + increment + len(contains)):
+        check = formatter.format(i)
+        if check not in contains:
+            return check
+
+    # this should really never happen since we looped
+    # through the full length of contains
+    raise ValueError('Unable to establish unique name!')
